'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

exports.default = function (_ref) {
  var t = _ref.types;

  var buildPropertyAssignment = function buildPropertyAssignment(objName, propName, valueNode) {
    return t.assignmentExpression('=', t.memberExpression(t.identifier(objName), t.identifier(propName)), valueNode);
  };

  function visitAssignmentExpression(path, renames) {
    var _path$node = path.node;
    var operator = _path$node.operator;
    var memberExpression = _path$node.left;
    var valueExpression = _path$node.right;

    if (operator !== '=' || !t.isMemberExpression(memberExpression)) {
      return;
    }
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      var _loop = function _loop() {
        var object = _step.value;

        if (!t.isIdentifier(memberExpression.object, { name: object })) {
          return 'continue';
        }
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = Object.keys(renames[object])[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var property = _step2.value;

            if (!t.isIdentifier(memberExpression.property, { name: property })) {
              continue;
            }

            var _concat = [].concat(renames[object][property]);

            var _concat2 = _toArray(_concat);

            var to = _concat2[0];

            var aliases = _concat2.slice(1);

            var renamedAssignment = buildPropertyAssignment(object, to, valueExpression);
            var chainedAliasAssignments = aliases.reduce(function (chained, alias) {
              return buildPropertyAssignment(object, alias, chained);
            }, renamedAssignment);
            path.replaceWith(t.expressionStatement(chainedAliasAssignments));
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      };

      for (var _iterator = Object.keys(renames)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _ret = _loop();

        if (_ret === 'continue') continue;
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  return {
    visitor: {
      AssignmentExpression: function AssignmentExpression(path, _ref2) {
        var opts = _ref2.opts;

        if (!opts.process || opts.process.includes('inline')) {
          visitAssignmentExpression(path, opts.renames);
        }
      },

      Program: {
        exit: function exit(program, _ref3) {
          var opts = _ref3.opts;

          if (opts.process && opts.process.includes('post')) {
            program.traverse({
              AssignmentExpression: function AssignmentExpression(path) {
                visitAssignmentExpression(path, opts.renames);
              }
            });
          }
        }
      }
    }
  };
};